%%clear all

%init

clear sound;


rng('default');


%init varicode
varicode_init;

%global settings
Fs=48000;%sampling rate

%psk31 settings
bitspersymbol=1;
symbolrate=31.25;%rate per second that symbols are sent
carrierfreq=1000;%carrier frequency audio sound for fm

%fsk settings
bitspersymbol=1;
M = 2;        % Modulation order
freqsep = 1000;  % Frequency separation (Hz)
symbolrate=1200;
carrierfreq=1700;
nsamp = Fs/symbolrate;    % Number of samples per symbol

%symbol timing
oversample_factor=nsamp;
hTimingSync = comm.GardnerTimingSynchronizer('SamplesPerSymbol',oversample_factor,'ErrorUpdateGain',0.001);
agc = comm.AGC;
agc.MaximumGain=1000;
agc.GainOutputPort=true;
agc.DetectorMethod='Square Law';
agc.StepSize=0.12;
agc.UpdatePeriod=12;
smybolFilt = designfilt('lowpassfir', 'FilterOrder', 80, 'HalfPowerFrequency', symbolrate*2, 'SampleRate', Fs);
DELAY = dsp.Delay;

%FM settings
Fc = 12000; % Carrier frequency of fm signal
dev = 5000; % Frequency deviation of fm signal
bpFilt = designfilt('bandpassfir','FilterOrder',round(0.5*Fs/480),'CutoffFrequency1',Fc-1.1*dev,'CutoffFrequency2',Fc+1.1*dev,'SampleRate',Fs);%FM descrim bpf

fb=symbolrate*bitspersymbol;%bit rate

[upfactor,downfactor] = rat(Fs/symbolrate);
actalsymbolraterate=(downfactor/upfactor*Fs);

rrcfilter_h = rcosdesign(0.5,3,upfactor,'sqrt');

hModulator = comm.DBPSKModulator;%PSK31 is actually DBPSK
hModulator.PhaseRotation=pi;%PSK31 0--> 180 rotation 1-->0 rotation
hDeModulator = comm.DBPSKDemodulator;
hDeModulator.PhaseRotation=pi;

hwt = dsp.SineWave(1, carrierfreq);
hwt.SampleRate=Fs;
hwt.ComplexOutput=true;



hError = comm.ErrorRate('ComputationDelay',0);

%end of init

message='  If you specify an incomplete or inconsistent set of name-value pairs at the command line, designfilt offers to open a Filter Design Assistant. The assistant helps you design the filter and pastes the corrected MATLAB code on the command line. ';

message= fileread('LDPC_DQPSK31.m');
message=sprintf('\n%s\n',message);
message_bits=vari_encode(message);

% % bits=4000;
% % message_bits=randi([0 1],bits,1);

BERs=zeros(100,1);
EbNos=zeros(100,1);
CNos=zeros(100,1);
i=1;
for current_ebno_value=[0,4,7,10,13,14,15,16,17,18:25]
    
    hError.reset();%clear BER calculator

    for itteration=1:5%(2+3*current_ebno_value)
        
        
        message_bits=vari_encode(message);
% %    message_bits=randi([0 1],bits,1);

        %fsk mod
        moddata= fskmod(1-message_bits,M,freqsep,nsamp,Fs);
        
        %raise frequency to passband
        hwt.release();
        hwt.SamplesPerFrame=numel(moddata);
        hwt.Frequency=-carrierfreq;
        csig=hwt.step().*moddata;
        sig=sqrt(2)*real(csig);
        
        %FM transmitter
        fmmodulationpercentage=1;% max FM modulation percentage is 1 
        sig=sig./max(abs(sig));
        sig=sig*fmmodulationpercentage;
        sig = fmmod(sig,Fc,Fs,dev);
        
        EbNo=current_ebno_value;%wanted EbNo in dB
        
        %calculate variance of noise for AWGN given wanted EbNo
        Eb=mean(sig.^2)/fb;
        No=Eb/(10^(EbNo/10));
        varn=No*(Fs/2);
        noise = normrnd(0,sqrt(varn),numel(sig),1);
        
% %         %calculate EbNo to check
% %         C=mean(sig.^2);%signal power
% %         Eb=C/(symbolrate*bitspersymbol);%Energy per bit
% %         No=mean(noise.^2)/(Fs/2);%Noise Power density
% %         EbNo=10*log10(Eb/No);%Energy used to send 1 bit comparied to the noise floor
% %         fprintf('EbNo=%f\n',EbNo);
C=bandpower(sig);
N=bandpower(noise);
Cno=10*log10(bandpower(sig)/(bandpower(noise)/(Fs/2)));
EbNoAct=Cno-10*log10(31.25);


        sig=sig+noise;
        
        %FM radio
        sig=filter(bpFilt,sig);
        sig = fmdemod(sig,Fc,Fs,dev);
        
        maxvolume=0.1;
        scale = 1/sqrt(mean(abs(sig).^2));
        sig=scale*sig*maxvolume;
        
        
% %     sound(sig,Fs);
% %     return

        hwt.release();
        hwt.SamplesPerFrame=numel(sig);
        hwt.Frequency=-carrierfreq;
        csig=hwt.step().*sig;%drop frequency to baseband
        
        %symbol timing
        DELAY.release;
        csig=[csig;zeros(agc.UpdatePeriod-mod(numel(csig),agc.UpdatePeriod),1)];
        [csig,agcgain]=agc.step(csig);
        csig=[csig;zeros(hTimingSync.SamplesPerSymbol-mod(numel(csig),hTimingSync.SamplesPerSymbol),1)];
        fcsig=filter(smybolFilt,csig);
        [~,PHASE] = step(hTimingSync,fcsig);
        PHASE(1:(nsamp*10))=[];
        PHASE=PHASE-smybolFilt.FilterOrder/2;
        PHASE=2*pi*PHASE/nsamp;
        PHASE=unwrap(PHASE);
        PHASE=nsamp*PHASE/(2*pi);
        while(mean(PHASE)<0)
            PHASE=PHASE+nsamp;
        end        
        DELAY.Length=round(mean(PHASE));
        csig=DELAY.step(csig);
        
        
        
        rx_message_bits= fskdemod(csig,M,freqsep,nsamp,Fs);
        
%        return
        
% %         vari_decode(rx_message_bits)
% %         return
        
        %align tx and rx messages and trim
        D = finddelay(message_bits,rx_message_bits);
        %D=2;
        rx_message_bits(1:D)=[];
        rx_message_bits(numel(message_bits)+1:end)=[];
        message_bits(numel(rx_message_bits)+1:end)=[];
        message_bits(1:32)=[];
        rx_message_bits(1:32)=[];
        message_bits(end-32:end)=[];
        rx_message_bits(end-32:end)=[];

        %update BER
        errors = step(hError, message_bits, rx_message_bits);

        fprintf('.');
% %         
% %         sumis=sum(xor(message_bits, rx_message_bits))
        
    end
    
    fprintf('\n');
    
    BERs(i)=errors(1);
    EbNos(i)=current_ebno_value;
    CNos(i)=Cno;
    
    fprintf('EbNo=%f BER=%f CNos=%f PSK31_Theory=%f\n',EbNos(i),BERs(i),CNos(i),berawgn(EbNos(i), 'dpsk', 2));
    
    
    
    i=i+1;
    
    %return;
end
BERs(i:end)=[];
EbNos(i:end)=[];
CNos(i:end)=[];
figure;
semilogy(EbNos,BERs,'o');
hold on;
BER_theory = berawgn(EbNos, 'dpsk', 2);
semilogy(EbNos,BER_theory,'r');
xlabel('EbNo (dB)');
ylabel('BER');
ylim([0.0000099 0.5]);
grid on;
hold off;

figure;
semilogy(CNos,BERs,'o');
hold on;
BER_theory = berawgn(EbNos, 'dpsk', 2);
semilogy(EbNos+10*log10(31.25),BER_theory,'r');
xlabel('CNo (dB-Hz)');
ylabel('BER');
ylim([0.0000099 0.5]);
grid on;
hold off;

return


