%clear all

%init

clear sound;


rng('default');


%init varicode
varicode_init;

%global settings
Fs=48000;%sampling rate

%FM settings
Fc = 12000; % Carrier frequency of fm signal
dev = 5000; % Frequency deviation of fm signal
bpFilt = designfilt('bandpassfir','FilterOrder',round(0.5*Fs/480),'CutoffFrequency1',Fc-1.1*dev,'CutoffFrequency2',Fc+1.1*dev,'SampleRate',Fs);%FM descrim bpf


%dbpsk31 settings
symbolrate=31.25;%rate per second that symbols are sent not bits
carrierfreq=1000;%carrier frequency audio sound
hModulator = comm.DBPSKModulator;%PSK31 is actually DBPSK
hModulator.PhaseRotation=pi;%PSK31 0--> 180 rotation 1-->0 rotation
hDeModulator = comm.DBPSKDemodulator;
hDeModulator.PhaseRotation=pi;
bitspersymbol=1;
oversample_factor=24;
hTimingSyncErrorUpdateGain=0.001;
agcUpdatePeriod=12;
agcStepSize=0.2;

%dbpsk1200 settings
% % % symbolrate=1200;%rate per second that symbols are sent not bits
% % % carrierfreq=1000;%carrier frequency audio sound
% % % hModulator = comm.DBPSKModulator;%PSK31 is actually DBPSK
% % % hModulator.PhaseRotation=pi;%PSK31 0--> 180 rotation 1-->0 rotation
% % % hDeModulator = comm.DBPSKDemodulator;
% % % hDeModulator.PhaseRotation=pi;
% % % bitspersymbol=1;
% % % oversample_factor=20;
% % % hTimingSyncErrorUpdateGain=0.001;
% % % agcUpdatePeriod=12;
% % % agcStepSize=0.2;

%dqpsk1200 settings
% % % symbolrate=600;%rate per second that symbols are sent not bits
% % % carrierfreq=1000;%carrier frequency audio sound
% % % hModulator = comm.DPSKModulator;%DQPSK
% % % hModulator.PhaseRotation=pi/2;
% % % hModulator.ModulationOrder=4;
% % % hModulator.BitInput=true;
% % % hDeModulator = comm.DPSKDemodulator;
% % % hDeModulator.PhaseRotation=pi/2;
% % % hDeModulator.ModulationOrder=4;
% % % hDeModulator.BitOutput=true;
% % % bitspersymbol=2;
% % % oversample_factor=20;
% % % hTimingSyncErrorUpdateGain=0.001;
% % % agcUpdatePeriod=12;
% % % agcStepSize=0.2;

fb=symbolrate*bitspersymbol;%bit rate

%symbol timing
hTimingSync = comm.GardnerTimingSynchronizer('SamplesPerSymbol',oversample_factor,'ErrorUpdateGain',hTimingSyncErrorUpdateGain);
agc = comm.AGC;
agc.MaximumGain=1000;
agc.GainOutputPort=true;
agc.DetectorMethod='Square Law';
agc.StepSize=agcStepSize;
agc.UpdatePeriod=agcUpdatePeriod;

[upfactor,downfactor] = rat(Fs/symbolrate);
actalsymbolraterate=(downfactor/upfactor*Fs);

rrcfilter_h = rcosdesign(0.5,3,upfactor,'sqrt');

hwt = dsp.SineWave(1, carrierfreq);
hwt.SampleRate=Fs;
hwt.ComplexOutput=true;

hError = comm.ErrorRate('ComputationDelay',0);

%end of init

message=' If you specify an incomplete or inconsistent set of name-value pairs at the command line, designfilt offers to open a Filter Design Assistant. The assistant helps you design the filter and pastes the corrected MATLAB code on the command line. ';
message= fileread('LDPC_DQPSK31.m');
message=sprintf('\n%s\n',message);
message_bits=vari_encode(message);


BERs=zeros(100,1);
EbNos=zeros(100,1);
CNos=zeros(100,1);
i=1;
for current_ebno_value=[0,4,7,10,13,14,15,16,17,18:25]
    
    hError.reset();%clear BER calculator

    for itteration=1:6%(2+3*current_ebno_value)
        

        
        
        %message_bits=vari_encode(message);

        %smaller random msg is needed for 31.25bps
    bits=3100;
    message_bits=randi([0 1],bits,1);

    
        hModulator.release();
        moddata=hModulator.step(message_bits);%modulate
        moddata_filtered = upfirdn(moddata,rrcfilter_h,upfactor,downfactor);%upsampling
        
        %raise frequency to passband
        hwt.release();
        hwt.Frequency=carrierfreq;
        hwt.SamplesPerFrame=numel(moddata_filtered);
        csig=hwt.step().*moddata_filtered;
        sig=sqrt(2)*real(csig);
        
% %      
% %      sound(real(csig),Fs);
% %      return
% %         
        fmmodulationpercentage=1;% max FM modulation percentage is 1 
        sig=sig./max(abs(sig));
        sig=sig*fmmodulationpercentage;
        sig = fmmod(sig,Fc,Fs,dev);
        
        EbNo=current_ebno_value;%wanted EbNo in dB
        
        %calculate variance of noise for AWGN given wanted EbNo
        Eb=mean(sig.^2)/fb;
        No=Eb/(10^(EbNo/10));
        varn=No*(Fs/2);
        noise = normrnd(0,sqrt(varn),numel(sig),1);
        
% %         %calculate EbNo to check
% %         C=mean(sig.^2);%signal power
% %         Eb=C/(symbolrate*bitspersymbol);%Energy per bit
% %         No=mean(noise.^2)/(Fs/2);%Noise Power density
% %         EbNo=10*log10(Eb/No);%Energy used to send 1 bit comparied to the noise floor
% %         fprintf('EbNo=%f\n',EbNo);
C=bandpower(sig);
N=bandpower(noise);
Cno=10*log10(bandpower(sig)/(bandpower(noise)/(Fs/2)));
EbNoAct=Cno-10*log10(31.25);


        sig=sig+noise;
        
        %FM radio
        sig=filter(bpFilt,sig);
        sig = fmdemod(sig,Fc,Fs,dev);
        
        maxvolume=0.1;
        scale = 1/sqrt(mean(abs(sig).^2));
        sig=scale*sig*maxvolume;
    
        
% %   sound(sig,Fs);
% %   return

        %drop frequency to baseband
        hwt.release();
        hwt.Frequency=-carrierfreq;
        hwt.SamplesPerFrame=numel(sig);
        csig=hwt.step().*sig;
        demoddata_filtered = upfirdn(csig,rrcfilter_h,downfactor,upfactor/oversample_factor);%downsampling

        %symbol timeing
        demoddata_filtered=[demoddata_filtered;zeros(agc.UpdatePeriod-mod(numel(demoddata_filtered),agc.UpdatePeriod),1)];
        [demoddata_filtered,agcgain]=agc.step(demoddata_filtered);
        demoddata_filtered=[demoddata_filtered;zeros(hTimingSync.SamplesPerSymbol-mod(numel(demoddata_filtered),hTimingSync.SamplesPerSymbol),1)];
        demoddata_filtered = step(hTimingSync,demoddata_filtered);

        rx_message_bits=hDeModulator.step(demoddata_filtered);
        
% %         vari_decode(rx_message_bits)
% %         return        
        
        %align tx and rx messages and trim
        D = finddelay(message_bits,rx_message_bits);
        %D=5;
        rx_message_bits(1:D)=[];
        rx_message_bits(numel(message_bits)+1:end)=[];
        message_bits(numel(rx_message_bits)+1:end)=[];
        message_bits(1:round(3*fb))=[];%allow two secs for agc and symbol timing to kick in
        rx_message_bits(1:round(3*fb))=[];
        message_bits(end-round(fb):end)=[];%remove last second
        rx_message_bits(end-round(fb):end)=[];
        
        %update BER       
        errors = step(hError, message_bits, rx_message_bits);
        
        fprintf('.');
  
        
    end
    
    fprintf('\n');
    
    BERs(i)=errors(1);
    EbNos(i)=current_ebno_value;
    CNos(i)=Cno;
    
    fprintf('EbNo=%f BER=%f CNos=%f PSK31_Theory=%f\n',EbNos(i),BERs(i),CNos(i),berawgn(EbNos(i), 'dpsk', 2));
    
    
    
    i=i+1;
    
    %return;
end
BERs(i:end)=[];
EbNos(i:end)=[];
CNos(i:end)=[];
figure;
semilogy(EbNos,BERs,'o');
hold on;
BER_theory = berawgn(EbNos, 'dpsk', 2);
semilogy(EbNos,BER_theory,'r');
xlabel('EbNo (dB)');
ylabel('BER');
ylim([0.0000099 0.5]);
xlim([0 25]);
grid on;
hold off;

return;

figure;
semilogy(CNos,BERs,'o');
hold on;
BER_theory = berawgn(EbNos, 'dpsk', 2);
semilogy(EbNos+10*log10(31.25),BER_theory,'r');
xlabel('CNo (dB-Hz)');
ylabel('BER');
ylim([0.0000099 0.5]);
hold off;

return

%----------------



figure;
semilogy(EbNos,BERs_fsk1200,'ob');
hold on;
semilogy(EbNos,BERs_dbpsk1200,'sg');
semilogy(EbNos,BERs_dqpsk1200,'<k');
semilogy(EbNos,BERs_dbpsk31,'>c');
BER_theory = berawgn(EbNos, 'dpsk', 2);
semilogy(EbNos,BER_theory,'r');
ylim([0.0000099 0.5]);
xlabel('EbNo (dB)');
ylabel('BER');
legend('FSK1200/NBFM','DBPSK1200/NBFM','DQPSK1200/NBFM','DBPSK31/NBFM','DBPSK/SSB');
title('Modulation efficiency, BER versus EbNo');
hold off;


CNOs_fsk1200=EbNos+10*log10(1200);
CNOs_dbpsk1200=EbNos+10*log10(1200);
CNOs_dqpsk1200=EbNos+10*log10(1200);
CNOs_dbpsk31=EbNos+10*log10(31.25);

figure;
semilogy(CNOs_fsk1200,BERs_fsk1200,'ob');
hold on;
semilogy(CNOs_dbpsk1200,BERs_dbpsk1200,'sg');
semilogy(CNOs_dqpsk1200,BERs_dqpsk1200,'<k');
semilogy(CNOs_dbpsk31,BERs_dbpsk31,'>c');
BER_theory = berawgn(EbNos, 'dpsk', 2);
semilogy(EbNos+10*log10(31.25),BER_theory,'r');
ylim([0.0000099 0.5]);
xlabel('CNo (dB-Hz)');
ylabel('BER');
legend('FSK1200/NBFM','DBPSK1200/NBFM','DQPSK1200/NBFM','DBPSK31/NBFM','DBPSK31/SSB');
title('On air transmitter power needed, BER versus CNo');
hold off;
